import hashlib


def calculate_checksum(file_path):
    hasher = hashlib.md5()
    with open(file_path, "rb") as f:
        for chunk in iter(lambda: f.read(4096), b""):
            hasher.update(chunk)
    return hasher.hexdigest()


# def calculate_checksum(file_path):
#     sha1_hash = hashlib.sha1()
#     with open(file_path, "rb") as file:
#         for chunk in iter(lambda: file.read(4096), b""):
#             sha1_hash.update(chunk)
#     return sha1_hash.hexdigest()



from pydantic import BaseModel
from src.utils import calculate_checksum


class IngestLargeDataset:
    def __init__(self, repo, logger):
        self.repo = repo
        self.logger = logger

    class Inputs(BaseModel):
        name: str
        path: str = None
        user: dict

    class Outputs(BaseModel):
        dataset: dict

    def __call__(self, inputs: Inputs) -> Outputs:
        data, error = self.repo.retrieve_dataset(inputs.name)
        if data:
            raise Exception("Dataset already exists")
        # allow only zip files
        if not inputs.path.endswith(".zip"):
            raise Exception("Only zip files are allowed")
        self.logger("Computing checksum...")
        checksum = calculate_checksum(inputs.path)
        self.logger(checksum)
        self.logger("Ingesting dataset...")
        id_token = inputs.user["id_token"]
        dataset_id, upload_id, parts = self.repo.prepare_large_upload(
            inputs.name, id_token, checksum
        )
        self.repo.ingest_large_dataset(
            inputs.path, upload_id, dataset_id, id_token, parts
        )
        self.logger("\nCompleting upload...")
        data, error = self.repo.complete_upload(
            inputs.name, id_token, upload_id, dataset_id, checksum
        )
        if error:
            raise Exception(error)
        self.logger("Done")
        return self.Outputs(dataset=data)



from .stac.dataframe import STACDataFrame  # , read_stac

# from .stac.stac import STACGenerator
# from .formatters import SHFolderFormatter
# from .stac.utils import format_time_acquired
# from .stac.parsers import STACIdParser, StructuredParser, UnestructuredParser



from .client import SHClient
from .utils import (SHParametersFeature, 
                    EvalScripts,
                    sentinel_1_download_parameters,
                    sentinel_2_download_parameters,
                    dem_download_parameters)


from pydantic import BaseModel


class IngestDataset:
    def __init__(self, repo, logger):
        self.repo = repo
        self.logger = logger

    class Inputs(BaseModel):
        name: str
        description: str
        path: str = None
        user: dict

    class Outputs(BaseModel):
        dataset: dict

    def __call__(self, inputs: Inputs) -> Outputs:
        # allow only zip files
        if not inputs.path.endswith(".zip"):
            raise Exception("Only zip files are allowed")
        self.logger("Ingesting dataset...")
        data, error = self.repo.ingest_dataset(
            inputs.name, inputs.description, inputs.path, inputs.user["id_token"]
        )
        # response = self.repo.ingest_large_dataset(inputs.name, inputs.description, inputs.path, inputs.user['id_token'])
        if error:
            raise Exception(error)
        self.logger("Done")
        return self.Outputs(dataset=data)



from ...repos import APIRepo
from .RetrieveDatasets import RetrieveDatasets
from .RetrieveDataset import RetrieveDataset
from .DownloadDataset import DownloadDataset
from .IngestDataset import IngestDataset
from .IngestLargeDataset import IngestLargeDataset
from .IngestLargeDatasetParallel import IngestLargeDatasetParallel
from .UpdateDataset import UpdateDataset


def retrieve_datasets():
    api_repo = APIRepo()
    retrieve = RetrieveDatasets(api_repo)
    inputs = retrieve.Inputs()
    outputs = retrieve(inputs)
    return outputs.datasets


def retrieve_dataset(name):
    api_repo = APIRepo()
    retrieve = RetrieveDataset(api_repo)
    inputs = retrieve.Inputs(name=name)
    outputs = retrieve(inputs)
    return outputs.dataset


def download_dataset(name, path, user, logger):
    dataset = retrieve_dataset(name)
    dataset_id = dataset["id"]
    checksum = dataset["checksum"]
    api_repo = APIRepo()
    download = DownloadDataset(api_repo, logger)
    inputs = download.Inputs(
        dataset=dataset_id, checksum=checksum, path=path, user=user
    )
    outputs = download(inputs)
    return outputs.dst_path


def ingest_dataset(name, description, path, user, logger):
    api_repo = APIRepo()
    ingest = IngestDataset(
        api_repo,
    )
    inputs = ingest.Inputs(name=name, description=description, path=path, user=user)
    outputs = ingest(inputs)
    return outputs.dataset


def ingest_large_dataset(name, path, user, logger):
    api_repo = APIRepo()
    ingest = IngestLargeDataset(api_repo, logger)
    inputs = ingest.Inputs(name=name, path=path, user=user)
    outputs = ingest(inputs)
    return outputs.dataset


def ingest_large_dataset_parallel(name, path, user, p, logger):
    api_repo = APIRepo()
    ingest = IngestLargeDatasetParallel(api_repo, logger)
    inputs = ingest.Inputs(name=name, path=path, user=user, threads=p)
    outputs = ingest(inputs)
    return outputs.dataset


def update_dataset(name, path, user, logger):
    api_repo = APIRepo()
    ingest = UpdateDataset(api_repo, logger)
    inputs = ingest.Inputs(name=name, path=path, user=user)
    outputs = ingest(inputs)
    return outputs.dataset



import typer
import os
import sys

# Add the eotdl_cli directory to the Python path
eotdl_cli_dir = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(eotdl_cli_dir))

from commands import auth, datasets

app = typer.Typer()
app.add_typer(auth.app, name="auth")
app.add_typer(datasets.app, name="datasets")

if __name__ == "__main__":
    app()



"""
STAC utils
"""

import pystac

from datetime import datetime
from dateutil import parser
from pandas import isna
from numpy import nan
from typing import Union


def format_time_acquired(dt: Union[str, datetime]) -> str:
    """
    Format the date time to the required format for STAC

    :param dt: date time to format
    """
    dt_str = parser.parse(dt).strftime("%Y-%m-%dT%H:%M:%S.%f")

    # convert the string to datetime object
    dt_obj = datetime.strptime(dt_str, "%Y-%m-%dT%H:%M:%S.%f")

    return dt_obj


def count_ocurrences(text: str, text_list: list) -> int:
    """
    Count the number of ocurrences of a string in a list of strings

    :param text: string to count the ocurrences
    """
    count = 0
    for string in text_list:
        if text in string:
            count += 1
    return count


def convert_df_geom_to_shape(row):
    """
    Convert the geometry of a dataframe row to a shapely shape

    :param row: row of a dataframe
    """
    from shapely.geometry import shape

    if not isna(row["geometry"]):
        geo = shape(row["geometry"])
        wkt = geo.wkt
    else:
        wkt = "POLYGON EMPTY"

    return wkt


def get_all_children(obj: pystac.STACObject) -> list:
    """
    Get all the children of a STAC object

    :param obj: STAC object
    """
    children = []

    # Append the current object to the list
    children.append(obj.to_dict())

    # Collections
    collections = list(obj.get_collections())

    for collection in collections:
        children.append(collection.to_dict())

    # Items
    items = obj.get_items()
    for item in items:
        children.append(item.to_dict())

    # Items from collections
    for collection in collections:
        items = collection.get_items()
        for item in items:
            children.append(item.to_dict())

    return children



import typer
from src.usecases.datasets import (
    retrieve_datasets,
    download_dataset,
    update_dataset,
    ingest_large_dataset,
    ingest_large_dataset_parallel,
)
from src.usecases.auth import auth
from typing import Optional

app = typer.Typer()


@app.command()
def list():
    """
    List all datasets
    """
    datasets = retrieve_datasets()
    typer.echo(datasets)


@app.command()
def get(name: str, path: str = None):
    """
    Download a dataset

    name: Name of the dataset
    path: Path to download the dataset to
    """
    try:
        user = auth()
        dst_path = download_dataset(name, path, user, typer.echo)
        typer.echo(f"Dataset {name} downloaded to {dst_path}")
    except Exception as e:
        typer.echo(e)


@app.command()
def ingest(
    path: str,
    name: str,
    # p: Optional[int] = 0,
):
    """
    Ingest a dataset

    path: Path to dataset to ingest
    n: Name of the dataset
    """
    try:
        user = auth()
        # if p:
        #     ingest_large_dataset_parallel(name, path, user, p, typer.echo)
        ingest_large_dataset(name, path, user, typer.echo)
        typer.echo(f"Dataset {name} ingested")
    except Exception as e:
        typer.echo(e)


@app.command()
def update(
    name: str,
    path: str,
):
    """
    Update a dataset

    name: Name of the dataset
    path: Path to dataset to ingest
    """
    try:
        user = auth()
        update_dataset(name, path, user, typer.echo)
        typer.echo(f"Dataset {name} updated")
    except Exception as e:
        typer.echo(e)


if __name__ == "__main__":
    app()



# from .stac import STACGenerator
# from .utils import format_time_acquired
# from .parsers import STACIdParser, StructuredParser, UnestructuredParser
# from .dataframe import STACDataFrame, read_stac



from pydantic import BaseModel

class IsLogged:
    def __init__(self, repo):
        self.repo = repo

    class Inputs(BaseModel):
        pass 

    class Outputs(BaseModel):
        user: dict = None

    def __call__(self, inputs: Inputs) -> Outputs:
        user = self.repo.load_creds()
        return self.Outputs(user=user)


import requests
from tqdm import tqdm
from pathlib import Path
import os
from concurrent.futures import ThreadPoolExecutor
import time
import multiprocessing
import hashlib


class APIRepo:
    def __init__(self, url=os.getenv("EOTDL_API_URL", "https://api.eotdl.com/")):
        self.url = url

    def login(self):
        return requests.get(self.url + "auth/login")

    def token(self, code):
        return requests.get(self.url + "auth/token?code=" + code)

    def logout_url(self):
        response = requests.get(self.url + "auth/logout")
        return response.json()["logout_url"]

    def retrieve_datasets(self):
        return requests.get(self.url + "datasets").json()

    def retrieve_dataset(self, name):
        response = requests.get(self.url + "datasets?name=" + name)
        if response.status_code == 200:
            return response.json(), None
        return None, response.json()["detail"]

    def download_dataset(self, dataset_id, id_token, path):
        url = self.url + "datasets/" + dataset_id + "/download"
        headers = {"Authorization": "Bearer " + id_token}
        if path is None:
            path = str(Path.home()) + "/.etodl/datasets"
            os.makedirs(path, exist_ok=True)
        with requests.get(url, headers=headers, stream=True) as r:
            r.raise_for_status()
            total_size = int(r.headers.get("content-length", 0))
            block_size = 1024 * 1024 * 10
            progress_bar = tqdm(
                total=total_size, unit="iB", unit_scale=True, unit_divisor=1024
            )
            filename = r.headers.get("content-disposition").split("filename=")[1][1:-1]
            path = f"{path}/{filename}"
            with open(path, "wb") as f:
                for chunk in r.iter_content(block_size):
                    progress_bar.update(len(chunk))
                    if chunk:
                        f.write(chunk)
            progress_bar.close()
            return path

    def read_in_chunks(self, file_object, CHUNK_SIZE):
        while True:
            data = file_object.read(CHUNK_SIZE)
            if not data:
                break
            yield data

    def prepare_large_upload(self, name, id_token, checksum):
        url = self.url + "datasets/chunk?name=" + name + "&checksum=" + checksum
        response = requests.get(url, headers={"Authorization": "Bearer " + id_token})
        if response.status_code != 200:
            raise Exception(response.json()["detail"])
        data = response.json()
        dataset_id, upload_id, parts = (
            data["dataset_id"],
            data["upload_id"],
            data["parts"] if "parts" in data else [],
        )
        return dataset_id, upload_id, parts

    def get_chunk_size(self, content_size):
        # adapt chunk size to content size to avoid S3 limits (10000 parts, 500MB per part, 5TB per object)
        chunk_size = 1024 * 1024 * 10  # 10 MB (up to 100 GB, 10000 parts)
        if content_size >= 1024 * 1024 * 1024 * 100:  # 100 GB
            chunk_size = 1024 * 1024 * 100  # 100 MB (up to 1 TB, 10000 parts)
        elif content_size >= 1024 * 1024 * 1024 * 1000:  # 1 TB
            chunk_size = 1024 * 1024 * 500  # 0.5 GB (up to 5 TB, 10000 parts)
        return chunk_size

    def ingest_large_dataset(self, path, upload_id, dataset_id, id_token, parts):
        content_path = os.path.abspath(path)
        content_size = os.stat(content_path).st_size
        chunk_size = self.get_chunk_size(content_size)
        total_chunks = content_size // chunk_size
        url = self.url + "datasets/chunk"
        headers = {
            "Authorization": "Bearer " + id_token,
            "Upload-Id": upload_id,
            "Dataset-Id": dataset_id,
        }
        # upload chunks sequentially
        pbar = tqdm(
            self.read_in_chunks(open(content_path, "rb"), chunk_size),
            total=total_chunks,
        )
        index = 0
        parts_checkusms = []
        for chunk in pbar:
            part = index // chunk_size + 1
            offset = index + len(chunk)
            index = offset
            if part not in parts:
                headers["Part-Number"] = str(part)
                checksum = hashlib.md5(chunk).hexdigest()
                parts_checkusms.append(checksum)
                headers["Checksum"] = checksum
                file = {"file": chunk}
                r = requests.post(url, files=file, headers=headers)
                if r.status_code != 200:
                    return None, r.json()["detail"]
            pbar.set_description(
                "{:.2f}/{:.2f} MB".format(
                    offset / 1024 / 1024, content_size / 1024 / 1024
                )
            )
        pbar.close()
        return

    def complete_upload(self, name, id_token, upload_id, dataset_id, checksum):
        url = self.url + "datasets/complete"
        r = requests.post(
            url,
            json={"name": name, "checksum": checksum},
            headers={
                "Authorization": "Bearer " + id_token,
                "Upload-Id": upload_id,
                "Dataset-Id": dataset_id,
            },
        )
        if r.status_code != 200:
            return None, r.json()["detail"]
        return r.json(), None

    def update_dataset(self, name, path, id_token, checksum):
        # check that dataset exists
        data, error = self.retrieve_dataset(name)
        if error:
            return None, error
        # first call to get upload id
        dataset_id = data["id"]
        url = self.url + f"datasets/chunk/{dataset_id}?checksum={checksum}"
        response = requests.get(url, headers={"Authorization": "Bearer " + id_token})
        if response.status_code != 200:
            return None, response.json()["detail"]
        data = response.json()
        _, upload_id, parts = data["dataset_id"], data["upload_id"], data["parts"]
        # assert dataset_id is None
        content_path = os.path.abspath(path)
        content_size = os.stat(content_path).st_size
        url = self.url + "datasets/chunk"
        chunk_size = 1024 * 1024 * 100  # 100 MiB
        total_chunks = content_size // chunk_size
        headers = {
            "Authorization": "Bearer " + id_token,
            "Upload-Id": upload_id,
            "Dataset-Id": dataset_id,
        }
        # upload chunks sequentially
        pbar = tqdm(
            self.read_in_chunks(open(content_path, "rb"), chunk_size),
            total=total_chunks,
        )
        index = 0
        for chunk in pbar:
            offset = index + len(chunk)
            part = index // chunk_size + 1
            index = offset
            if part not in parts:
                headers["Part-Number"] = str(part)
                file = {"file": chunk}
                r = requests.post(url, files=file, headers=headers)
                if r.status_code != 200:
                    return None, r.json()["detail"]
            pbar.set_description(
                "{:.2f}/{:.2f} MB".format(
                    offset / 1024 / 1024, content_size / 1024 / 1024
                )
            )
        pbar.close()
        # complete upload
        url = self.url + "datasets/complete"
        r = requests.post(
            url,
            json={"checksum": checksum},
            headers={
                "Authorization": "Bearer " + id_token,
                "Upload-Id": upload_id,
                "Dataset-Id": dataset_id,
            },
        )
        if r.status_code != 200:
            return None, r.json()["detail"]
        return r.json(), None

    def ingest_large_dataset_parallel(
        self,
        path,
        upload_id,
        dataset_id,
        id_token,
        parts,
        threads,
    ):
        # Create thread pool executor
        max_workers = threads if threads > 0 else multiprocessing.cpu_count()
        executor = ThreadPoolExecutor(max_workers=max_workers)

        # Divide file into chunks and create tasks for each chunk
        offset = 0
        tasks = []
        content_path = os.path.abspath(path)
        content_size = os.stat(content_path).st_size
        chunk_size = self.get_chunk_size(content_size)
        total_chunks = content_size // chunk_size
        while offset < content_size:
            chunk_end = min(offset + chunk_size, content_size)
            part = str(offset // chunk_size + 1)
            if part not in parts:
                tasks.append((offset, chunk_end, part))
            offset = chunk_end

        # Define the function that will upload each chunk
        def upload_chunk(start, end, part):
            # print(f"Uploading chunk {start} - {end}", part)
            with open(content_path, "rb") as f:
                f.seek(start)
                chunk = f.read(end - start)
            checksum = hashlib.md5(chunk).hexdigest()
            response = requests.post(
                self.url + "datasets/chunk",
                files={"file": chunk},
                headers={
                    "Authorization": "Bearer " + id_token,
                    "Upload-Id": upload_id,
                    "Dataset-Id": dataset_id,
                    "Checksum": checksum,
                    "Part-Number": str(part),
                },
            )
            if response.status_code != 200:
                print(f"Failed to upload chunk {start} - {end}")
            return response

        # Submit each task to the executor
        with tqdm(total=total_chunks) as pbar:
            futures = []
            for task in tasks:
                future = executor.submit(upload_chunk, *task)
                future.add_done_callback(lambda p: pbar.update())
                futures.append(future)

            # Wait for all tasks to complete
            for future in futures:
                future.result()



"""
Module for managing the Sentinel Hub configuration and data access
"""
from os.path import join
from sentinelhub import (SHConfig, 
                         SentinelHubCatalog, 
                         BBox, 
                         bbox_to_dimensions, 
                         CRS, 
                         SentinelHubRequest, 
                         SentinelHubDownloadClient,
                         MimeType,
                         DataCollection)

from .utils import SHParametersFeature, EvalScripts

# Relate the DataCollection.api_id with the corresponding evalscript
# We will use it in the SHClient.request_data function
data_collection_evalscripts = {'sentinel-1-grd': EvalScripts.SENTINEL_1,
                               'sentinel-2-l2a': EvalScripts.SENTINEL_2,
                               'dem': EvalScripts.DEM}


class SHClient():
    """
    Client class to manage the Sentinel Hub Python interface.
    """

    def __init__(self, 
                 sh_client_id: str, 
                 sh_client_secret: str
                 ) -> None:
        """
        :param sh_client_id: User's OAuth client ID for Sentinel Hub service.
        :param sh_client_secret: User's OAuth client secret for Sentinel Hub service.
        """
        self.config = SHConfig()
        self.config.sh_client_id = sh_client_id
        self.config.sh_client_secret = sh_client_secret
        self.catalog = SentinelHubCatalog(config=self.config)

    def search_available_sentinel_data(self,
                                       parameters: SHParametersFeature
                                       ) -> list:
        """
        Search and return the available Sentinel data from a given DataCollection in a selected location and time interval. 

        :param parameters: ParametersFeature object with the needed parameters for the search. The neeeded parameters are:
                    - data_collection: Required.
                    - bbox: Required.
                    - time_interval: Required.
                    - fields: Required.
                    - filter: Optional.
        :return results: list with the obtained results.
        """
        search_iterator = self.catalog.search(
            parameters.data_collection,
            bbox=BBox(parameters.bounding_box, crs=CRS.WGS84),
            time=parameters.time_interval,
            filter=parameters.filter,
            fields=parameters.fields
        )

        results = list(search_iterator)

        return results

    def request_bulk_data(self, 
                    parameters: SHParametersFeature
                    ) -> list:
        """
        Get and return a list with the download information for every location. 
        This is the prefered way to download data, as in order to efficiently 
        download data for all requests in parallel, we extract the download 
        information and we pass it to a download client.

        :param parameters: ParametersFeature object with the needed parameters for the search. The neeeded parameters are:
                    - data_to_download: Required. Dictionary with the information about the data that the user wants to
                        download. It could have two formats:
                            - location_id : list(bounding_box)
                            - location_id : dict(bounding_box: list(bounding_box),
                                                time_interval: list(list(time_interval), list(time_interval) ...)
                                                )
                    - data_collection: Required.
                    - resolution: Required.
                    - data_folder: Required.

        :return process_request: list with the download information for every location
        """
        # Use exactly the needed service url to avoid errors
        self.config.sh_base_url = parameters.data_collection.service_url   
        
        process_requests = list()

        root_folder = parameters.data_folder

        for id, info in parameters.data_to_download.items():
            # We should distinct between the possible input formats
            if isinstance(info, tuple):
                bbox = BBox(info, crs=CRS.WGS84)
                time_interval = [None]
            elif isinstance(info, dict):   # Bulk download
                bbox = BBox(info['bounding_box'], crs=CRS.WGS84)
                time_interval = info['time_interval']
            parameters.bounding_box = bbox

            # Create a different data folder for each request
            _data_folder = join(root_folder, f'{parameters.data_collection.api_id}_{id}')

            for time in time_interval:
                # Add the date to the data folder name, if it exists
                data_folder = f'{_data_folder}_{time[1]}' if time else _data_folder
                # Add required parameters to the ParametersFeature, in order to
                # pass it to the request data function
                parameters.data_folder = data_folder
                parameters.time_interval = time
                # Do the request and append it to the requests list 
                request = self.request_data(parameters)
                process_requests.append(request)

        return process_requests
    
    def request_data(self, 
                    parameters: SHParametersFeature
                    ) -> list:
        """
        Request the specified Sentinel Hub data and return the SentinelHubRequest object

        :param parameters: ParametersFeature object with the needed parameters for the search. The neeeded parameters are:
                    - time_interval: Required.
                    - data_collection: Required.
                    - resolution: Required.
                    - data_folder: Required.
                    - bounding_box: Required.
                    - mosaicking_order: Optional.

        """
        return SentinelHubRequest(
                    data_folder=parameters.data_folder,
                    evalscript=data_collection_evalscripts[parameters.data_collection.api_id],
                    input_data=[
                        SentinelHubRequest.input_data(
                            data_collection=parameters.data_collection,
                            time_interval=parameters.time_interval,
                            mosaicking_order=parameters.mosaicking_order
                        )
                    ],
                    responses=[
                                SentinelHubRequest.output_response("default", MimeType.TIFF)
                    ],
                    bbox=parameters.bounding_box,
                    size=bbox_to_dimensions(parameters.bounding_box, parameters.resolution),
                    config=self.config,
                )

    def download_data(self,
                      requests: list
                      ) -> list:
        """
        Download data from Sentinel Hub Services using a list of requests. 
        This is the prefered way to download data, in order to efficiently 
        download data for all requests in parallel.
        
        :param requests: list with SentinelHubRequest objects representing each 
                        request to the Sentinel Hub Services.
        :return: list with the downloaded data
        """
        download_client = SentinelHubDownloadClient(config=self.config)
        download_requests = [request.download_list[0] for request in requests]
        # Download data with multiple threads
        data = download_client.download(download_requests)

        return data



from .tools import (get_images_by_location, 
                    calculate_average_coordinates_distance,
                    generate_new_locations_bounding_boxes,
                    get_available_data_by_location)


from pathlib import Path
import os 
import json 
import jwt

class AuthRepo():
    def __init__(self):
        self.algorithms = ['RS256']
        self.home = str(Path.home())
        self.creds_path = self.home + '/.etodl/creds.json'

    def save_creds(self, data):
        os.makedirs(self.home + '/.etodl', exist_ok=True)
        with open(self.creds_path, 'w') as f:
            json.dump(data, f)
        return self.creds_path
    
    def load_creds(self):
        if os.path.exists(self.creds_path):
            with open(self.creds_path, 'r') as f:
                creds = json.load(f)
            user = self.decode_token(creds)
            user['id_token'] = creds['id_token']
            return user
        return None
    
    def decode_token(self, token_data):
        return jwt.decode(token_data['id_token'], algorithms=self.algorithms, options={"verify_signature": False})
    
    def logout(self):
        os.remove(self.creds_path)



"""
Module for STAC extensions objects
"""

import pystac
from pystac.extensions.sar import SarExtension
from pystac.extensions.sar import FrequencyBand, Polarization
from pystac.extensions.eo import Band, EOExtension
from typing import Union


class STACExtensionObject:
    def __init__(self) -> None:
        super().__init__()
        self.properties = dict()

    def add_extension_to_object(
        self, obj: Union[pystac.Item, pystac.Asset]
    ) -> Union[pystac.Item, pystac.Asset]:
        """
        Add the extension to the given object

        :param obj: object to add the extension
        """
        pass


class SarExtensionObject(STACExtensionObject):
    def __init__(self) -> None:
        super().__init__()
        pass

    def add_extension_to_object(
        self, obj: Union[pystac.Item, pystac.Asset]
    ) -> Union[pystac.Item, pystac.Asset]:
        """
        Add the extension to the given object

        :param obj: object to add the extension
        """
        sar_ext = SarExtension.ext(obj, add_if_missing=True)
        if isinstance(obj, pystac.Item):
            polarizations = [Polarization.VV, Polarization.VH]
        elif isinstance(obj, pystac.Asset):
            polarizations_dict = {"VV": Polarization.VV, "VH": Polarization.VH}
            polarizations = [polarizations_dict[obj.title]]
        sar_ext.apply(
            instrument_mode="EW",
            polarizations=polarizations,
            frequency_band=FrequencyBand.C,
            product_type="GRD",
        )

        return obj


class EOS2ExtensionObject(STACExtensionObject):
    def __init__(self) -> None:
        super().__init__()
        self.bands = [
            Band.create(
                name="Aerosols",
                description="Coastal aerosol, 442.7 nm (S2A), 442.3 nm (S2B)",
                common_name="coastal",
            ),
            Band.create(
                name="Blue",
                description="Blue, 492.4 nm (S2A), 492.1 nm (S2B)",
                common_name="blue",
            ),
            Band.create(
                name="Green",
                description="Green, 559.8 nm (S2A), 559.0 nm (S2B)",
                common_name="green",
            ),
            Band.create(
                name="Red",
                description="Red, 664.6 nm (S2A), 665.0 nm (S2B)",
                common_name="red",
            ),
            Band.create(
                name="Red edge 1",
                description="Vegetation red edge, 704.1 nm (S2A), 703.8 nm (S2B)",
                common_name="rededge",
            ),
            Band.create(
                name="Red edge 2",
                description="Vegetation red edge, 740.5 nm (S2A), 739.1 nm (S2B)",
                common_name="rededge",
            ),
            Band.create(
                name="Red edge 3",
                description="Vegetation red edge, 782.8 nm (S2A), 779.7 nm (S2B)",
                common_name="rededge",
            ),
            Band.create(
                name="NIR",
                description="NIR, 832.8 nm (S2A), 833.0 nm (S2B)",
                common_name="nir",
            ),
            Band.create(
                name="Red edge 4",
                description="Narrow NIR, 864.7 nm (S2A), 864.0 nm (S2B)",
                common_name="nir08",
            ),
            Band.create(
                name="Water vapour",
                description="Water vapour, 945.1 nm (S2A), 943.2 nm (S2B)",
                common_name="nir09",
            ),
            Band.create(
                name="Cirrus",
                description="SWIR – Cirrus, 1373.5 nm (S2A), 1376.9 nm (S2B)",
                common_name="cirrus",
            ),
            Band.create(
                name="SWIR1",
                description="SWIR, 1613.7 nm (S2A), 1610.4 nm (S2B)",
                common_name="swir16",
            ),
            Band.create(
                name="SWIR2",
                description="SWIR, 2202.4 nm (S2A), 2185.7 nm (S2B)",
                common_name="swir22",
            ),
        ]

    def add_extension_to_object(
        self, obj: Union[pystac.Item, pystac.Asset]
    ) -> Union[pystac.Item, pystac.Asset]:
        """
        Add the extension to the given object

        :param obj: object to add the extension
        """
        if isinstance(obj, pystac.Asset):
            return
        # Add EO extension
        eo_ext = EOExtension.ext(obj, add_if_missing=True)
        # Add the existing bands from the rasters assets list
        eo_ext.apply(bands=self.bands)
        # Add common metadata
        obj.common_metadata.constellation = "Sentinel-2"
        obj.common_metadata.platform = "Sentinel-2"
        obj.common_metadata.instruments = ["Sentinel-2"]
        obj.common_metadata.gsd = 10  # TODO Where to obtain it?

        return obj


class DEMExtensionObject(STACExtensionObject):
    DEM_DATE_ACQUIRED = {
        "start_datetime": "2011-01-01T00:00:00Z",
        "end_datetime": "2015-01-07T00:00:00Z",
    }

    def __init__(self) -> None:
        super().__init__()


type_stac_extensions_dict = {
    "sar": SarExtensionObject(),
    "eo": EOS2ExtensionObject(),
    "dem": DEMExtensionObject(),
}



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['eotdl',
 'eotdl.access',
 'eotdl.access.sentinelhub',
 'eotdl.commands',
 'eotdl.curation',
 'eotdl.curation.stac',
 'eotdl.datasets',
 'eotdl.src',
 'eotdl.src.errors',
 'eotdl.src.repos',
 'eotdl.src.usecases',
 'eotdl.src.usecases.auth',
 'eotdl.src.usecases.datasets',
 'eotdl.tools',
 'eotdl.tools.sen12floods']

package_data = \
{'': ['*']}

install_requires = \
['pydantic>=1.10.6,<2.0.0',
 'pyjwt>=2.6.0,<3.0.0',
 'requests>=2.28.2,<3.0.0',
 'tqdm>=4.65.0,<5.0.0',
 'typer[all]>=0.7.0,<0.8.0']

entry_points = \
{'console_scripts': ['eotdl = eotdl.main:app']}

setup_kwargs = {
    'name': 'eotdl',
    'version': '2023.6.14.post2',
    'description': 'Earth Observation Training Data Lab',
    'long_description': '# eotdl \n\nThis is the main library for EOTDL.\n\n',
    'author': 'EarthPulse',
    'author_email': 'it@earthpulse.es',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
